/* Temporary internal state that avoids blocking tracking until getting DMP */
export var dmpState = {
  isDmpReady: undefined
};
/**
 * Check if DMP is consented by the user using boros API
 * @return {Promise<boolean>}
 */

export var checkIsDMPReady = function checkIsDMPReady() {
  return new Promise(function (resolve) {
    if (window.__borosTcf === undefined) return resolve();

    window.__borosTcf.push(function (api) {
      api('isDmpAccepted', function (_ref) {
        var success = _ref.success,
            value = _ref.value;
        dmpState.isDmpReady = success && value;
        return resolve(dmpState.isDmpReady);
      });
    });
  });
};
/**
 * Get if DMP is consented value
 * @return {Promise<boolean>}
 */

export var getIsDMPReady = function getIsDMPReady() {
  var isDmpReady = dmpState.isDmpReady; // if we have some value defined for isDmpReady
  // we returning it and don't check anything else

  if (isDmpReady !== undefined) return Promise.resolve(isDmpReady); // if we don't have any value, we must check async
  // but as we don't want to block anything yet
  // we assume we don't have the consent and return false

  return checkIsDMPReady();
};