import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { getConfig } from './config.js';
import { getUniversalId } from './universalId.js';
var ADOBE_ORG_ID = '05FF6243578784B37F000101@AdobeOrg';
var AUTHENTICATE_STATE = 1;
var DEFAULT_DEMDEX_VERSION = '3.3.0';
var TIME_BETWEEN_RETRIES = 15;
var TIMES_TO_RETRY = 80;
var TRACKING_SERVER = 'schibstedspain.d3.sc.omtrdc.net';
var SERVERS = {
  trackingServer: TRACKING_SERVER,
  trackingServerSecure: TRACKING_SERVER
};
var mcvid;

var syncMail = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(demdex) {
    var universalId;
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            universalId = getUniversalId();

            if (universalId) {
              demdex.setCustomerIDs({
                hashed_email: {
                  id: universalId,
                  authState: AUTHENTICATE_STATE
                }
              });
            }

          case 2:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function syncMail(_x) {
    return _ref.apply(this, arguments);
  };
}();

var getDemdex = function getDemdex() {
  return window.Visitor && window.Visitor.getInstance(ADOBE_ORG_ID, SERVERS);
};

var getMarketingCloudVisitorID = function getMarketingCloudVisitorID(demdex) {
  var mcvid = demdex && demdex.getMarketingCloudVisitorID();
  syncMail(demdex);
  return mcvid;
};

var getAdobeVisitorData = function getAdobeVisitorData() {
  var demdex = getDemdex() || {};
  var _demdex$version = demdex.version,
      version = _demdex$version === void 0 ? DEFAULT_DEMDEX_VERSION : _demdex$version;
  var trackingServer = SERVERS.trackingServer;
  return Promise.resolve({
    trackingServer: trackingServer,
    version: version
  });
};

export var getAdobeMarketingCloudVisitorIdFromWindow = function getAdobeMarketingCloudVisitorIdFromWindow() {
  if (mcvid) return Promise.resolve(mcvid);
  return new Promise(function (resolve) {
    function retry(retries) {
      if (retries === 0) return resolve('');
      var demdex = getDemdex();
      mcvid = getMarketingCloudVisitorID(demdex);
      return mcvid ? resolve(mcvid) : window.setTimeout(function () {
        return retry(--retries);
      }, TIME_BETWEEN_RETRIES);
    }

    retry(TIMES_TO_RETRY);
  });
};

var importVisitorApiAndGetAdobeMCVisitorID = function importVisitorApiAndGetAdobeMCVisitorID() {
  return import('./adobeVisitorApi.js').then(function () {
    mcvid = getAdobeMarketingCloudVisitorIdFromWindow();
    return mcvid;
  });
};

var getAdobeMCVisitorID = function getAdobeMCVisitorID() {
  var getCustomAdobeVisitorId = getConfig('getCustomAdobeVisitorId');

  if (typeof getCustomAdobeVisitorId === 'function') {
    return getCustomAdobeVisitorId();
  }

  return getConfig('importAdobeVisitorId') === true ? importVisitorApiAndGetAdobeMCVisitorID() : getAdobeMarketingCloudVisitorIdFromWindow();
};

export { getAdobeVisitorData, getAdobeMCVisitorID };