import _asyncToGenerator from "@babel/runtime/helpers/esm/asyncToGenerator";
import _extends from "@babel/runtime/helpers/esm/extends";
import _regeneratorRuntime from "@babel/runtime/regenerator";
import { syncPixels } from './syncPixels';
import { getConfig } from './config';
import { getAdobeMCVisitorID } from './adobeRepository.js';
import { checkGdprIsAccepted, getGdprPrivacyValue } from './tcf';
import { getIsDMPReady } from './adobeDMP';
/* Default properties to be sent on all trackings */

var DEFAULT_PROPERTIES = {
  platform: 'web'
};
/* Disabled integrations when no GDPR Privacy Value is true */

export var INTEGRATIONS_WHEN_NO_CONSENTS = {
  All: false
};
/* ServerSide Forwarding values */

var SSF = {
  enabled: 0,
  disabled: 1
};
/* Static anonymousId for users that has not accepted consents */

var SEGMENT_ID_USER_WITHOUT_CONSENTS = 'anonymous_user';
/**
 * Get default properties using the constant and the window.__mpi object if available
 * @returns {{[key:string]: any}} Default properties to attach to track
 */

export var getDefaultProperties = function getDefaultProperties() {
  return _extends({}, DEFAULT_PROPERTIES, getConfig('defaultProperties'));
};
/**
 * Get user traits from global analytics object and put in the object
 * @param {string} gdprPrivacyValue Determine if we have user consents
 * @returns {object} User traits with to add
 */

var getUserTraits = /*#__PURE__*/function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee(gdprPrivacyValue) {
    return _regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (checkGdprIsAccepted(gdprPrivacyValue)) {
              _context.next = 4;
              break;
            }

            window.analytics.reset();
            window.analytics.setAnonymousId(SEGMENT_ID_USER_WITHOUT_CONSENTS);
            return _context.abrupt("return", {
              anonymousId: SEGMENT_ID_USER_WITHOUT_CONSENTS
            });

          case 4:
            return _context.abrupt("return", new Promise(function (resolve) {
              window.analytics.ready(function () {
                var user = window.analytics.user();

                if (user.anonymousId() === SEGMENT_ID_USER_WITHOUT_CONSENTS) {
                  window.analytics.reset();
                }

                resolve({
                  anonymousId: user.anonymousId(),
                  userId: user.id()
                });
              });
            }));

          case 5:
          case "end":
            return _context.stop();
        }
      }
    }, _callee);
  }));

  return function getUserTraits(_x) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * Get all needed integrations depending on the gdprPrivacy value.
 * One of them is the AdobeMarketingCloudVisitorId for Adobe Analytics integration.
 * @param {string} gdprPrivacyValue Tell if user has consents to be tracked
 * @returns {Promise<object>} Integrations that we need to use on the track
 */


var getTrackIntegrations = /*#__PURE__*/function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2(gdprPrivacyValue) {
    var adobeAnalyticsIntegration, restOfIntegrations;
    return _regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            if (!checkGdprIsAccepted(gdprPrivacyValue)) {
              _context2.next = 7;
              break;
            }

            _context2.next = 3;
            return getAdobeMCVisitorID();

          case 3:
            _context2.t1 = _context2.sent;
            _context2.t0 = {
              marketingCloudVisitorId: _context2.t1
            };
            _context2.next = 8;
            break;

          case 7:
            _context2.t0 = true;

          case 8:
            adobeAnalyticsIntegration = _context2.t0;
            restOfIntegrations = checkGdprIsAccepted(gdprPrivacyValue) ? {} : INTEGRATIONS_WHEN_NO_CONSENTS; // if we don't have the user consents we remove all the integrations but Adobe Analytics

            return _context2.abrupt("return", _extends({}, restOfIntegrations, {
              'Adobe Analytics': adobeAnalyticsIntegration,
              'Amazon Lambda': true
            }));

          case 11:
          case "end":
            return _context2.stop();
        }
      }
    }, _callee2);
  }));

  return function getTrackIntegrations(_x2) {
    return _ref2.apply(this, arguments);
  };
}();
/**
 * Get data like traits and integrations to be added to the context object
 * @param {object} context Context object with all the actual info
 * @returns {Promise<object>} New context with all the previous info and the new one
 */


export var decorateContextWithNeededData = /*#__PURE__*/function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3(context) {
    var gdprPrivacyValue, isGdprAccepted, _yield$Promise$all, integrations, isDMPReady, userTraits;

    return _regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            if (context === void 0) {
              context = {};
            }

            _context3.next = 3;
            return getGdprPrivacyValue();

          case 3:
            gdprPrivacyValue = _context3.sent;
            isGdprAccepted = checkGdprIsAccepted(gdprPrivacyValue);
            _context3.next = 7;
            return Promise.all([getTrackIntegrations(gdprPrivacyValue), getIsDMPReady(), getUserTraits(gdprPrivacyValue)]);

          case 7:
            _yield$Promise$all = _context3.sent;
            integrations = _yield$Promise$all[0];
            isDMPReady = _yield$Promise$all[1];
            userTraits = _yield$Promise$all[2];
            return _context3.abrupt("return", _extends({}, context, !isGdprAccepted && {
              ip: '0.0.0.0'
            }, {
              // anonymizing ip if user has no consents
              gdpr_privacy: gdprPrivacyValue,
              integrations: _extends({}, context.integrations, integrations),
              traits: _extends({}, context.traits, {
                ssf: isDMPReady ? SSF.enabled : SSF.disabled
              }, userTraits)
            }));

          case 12:
          case "end":
            return _context3.stop();
        }
      }
    }, _callee3);
  }));

  return function decorateContextWithNeededData(_x3) {
    return _ref3.apply(this, arguments);
  };
}();
/**
 * The track method lets you record any actions your users perform.
 * @param {string} event The name of the event you’re tracking
 * @param {object} [properties] A dictionary of properties for the event.
 * @param {object} [context] A dictionary of options.
 * @param {function} [callback] A function executed after a short timeout, giving the browser time to make outbound requests first.
 * @returns {Promise}
 */

var track = function track(event, properties, context, callback) {
  if (context === void 0) {
    context = {};
  }

  return new Promise(function (resolve, reject) {
    var initTrack = /*#__PURE__*/function () {
      var _ref4 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee5() {
        var newContext, newProperties, newCallback;
        return _regeneratorRuntime.wrap(function _callee5$(_context5) {
          while (1) {
            switch (_context5.prev = _context5.next) {
              case 0:
                _context5.next = 2;
                return decorateContextWithNeededData(context);

              case 2:
                newContext = _context5.sent;

                /**
                 * @deprecated Now we use `defaultContextProperties` middleware
                 * and put the info on the context object
                 */
                newProperties = _extends({}, getDefaultProperties(), properties);

                newCallback = /*#__PURE__*/function () {
                  var _ref5 = _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee4() {
                    var _len,
                        args,
                        _key,
                        _yield$Promise$all2,
                        gdprPrivacyValue,
                        isDMPReady,
                        marketingCloudVisitorId,
                        _args4 = arguments;

                    return _regeneratorRuntime.wrap(function _callee4$(_context4) {
                      while (1) {
                        switch (_context4.prev = _context4.next) {
                          case 0:
                            for (_len = _args4.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
                              args[_key] = _args4[_key];
                            }

                            if (callback) callback.apply(void 0, args); // eslint-disable-line standard/no-callback-literal

                            _context4.next = 4;
                            return Promise.all([getGdprPrivacyValue(), getIsDMPReady()]);

                          case 4:
                            _yield$Promise$all2 = _context4.sent;
                            gdprPrivacyValue = _yield$Promise$all2[0];
                            isDMPReady = _yield$Promise$all2[1];

                            if (!(isDMPReady && checkGdprIsAccepted(gdprPrivacyValue))) {
                              _context4.next = 14;
                              break;
                            }

                            _context4.next = 10;
                            return getAdobeMCVisitorID();

                          case 10:
                            marketingCloudVisitorId = _context4.sent;
                            return _context4.abrupt("return", syncPixels(marketingCloudVisitorId).then(function () {
                              return resolve.apply(void 0, args);
                            })["catch"](reject));

                          case 14:
                            resolve();

                          case 15:
                          case "end":
                            return _context4.stop();
                        }
                      }
                    }, _callee4);
                  }));

                  return function newCallback() {
                    return _ref5.apply(this, arguments);
                  };
                }();

                window.analytics.track(event, newProperties, newContext, newCallback);

              case 6:
              case "end":
                return _context5.stop();
            }
          }
        }, _callee5);
      }));

      return function initTrack() {
        return _ref4.apply(this, arguments);
      };
    }();

    initTrack();
  });
};
/**
 * Associate your users and their actions to a recognizable userId and traits.
 * @param {string} userId Id to identify the user.
 * @param {object} traits A dictionary of traits you know about the user, like their email or name.
 * @param {object} [options] A dictionary of options.
 * @param {function} [callback] A function executed after a short timeout, giving the browser time to make outbound requests first.
 * @returns {Promise}
 */


var identify = function identify(userId, traits, options, callback) {
  return Promise.resolve(window.analytics.identify(userId, traits, options, callback));
};
/**
 * Record whenever a user sees a page of your website, along with any optional properties about the page.
 * @param {string} event The name of the event you’re tracking
 * @param {object=} properties A dictionary of properties for the event.
 * @param {object} [context] A dictionary of options.
 * @param {function} [callback] A function executed after a short timeout, giving the browser time to make outbound requests first.
 * @returns {Promise}
 */


var page = function page(event, properties, context, callback) {
  if (context === void 0) {
    context = {};
  }

  // we put a flag on context to know this track is a page
  context.isPageTrack = true; // just call track again but the with the proper context

  return track(event, properties, context, callback);
};
/**
 * Resets the id, including anonymousId, and clear traits for the currently identified user and group.
 * NOTE: Only clears the cookies and localStorage set by analytics.
 * @returns {Promise}
 */


var reset = function reset() {
  return Promise.resolve(window.analytics.reset());
};

export default {
  page: page,
  identify: identify,
  track: track,
  reset: reset
};